import json
import matplotlib.pyplot as plt
plt.ioff()
import torch
from bisect import bisect_left
from PyQt5.QtGui import QDoubleValidator, QIntValidator
from PyQt5.QtCore import Qt, QLocale, pyqtSignal, pyqtSlot  
from torchPIV.PlotterFunctions import Database, PIVparams, set_width
from PyQt5.QtWidgets import (
    QFrame,
    QLabel,
    QProgressBar,
    QPushButton,
    QVBoxLayout,
    QHBoxLayout,
    QLineEdit,
    QFileDialog,
    QTextEdit,
    QWidget,
    QComboBox,
    QLCDNumber,
    QSlider,
    QCheckBox,
)


class ListSlider(QSlider):
    elementChanged = pyqtSignal(int)

    def __init__(self, values=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setMinimum(0)
        self._values = []
        self.valueChanged.connect(self._on_value_changed)
        self.values = values or []

    @property
    def values(self):
        return self._values

    @values.setter
    def values(self, values: list):
        self._values = values
        maximum = max(0, len(self._values) - 1)
        self.setMaximum(maximum)
        self.setValue(0)

    @pyqtSlot(int)
    def _on_value_changed(self, index: int):
        value = self.values[index]
        self.elementChanged.emit(value)
    def set_value(self, value: str):
        index = bisect_left(self._values, float(value))
        self.setValue(index)
        return index



class Settings(QWidget):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__( *args, **kwargs)
        self.state = PIVparams()
        self.state.from_json()
        self.initUI()

    def initUI(self):
        hbox_top  = QHBoxLayout()
        hbox_bot  = QHBoxLayout()
        vbox_down = QVBoxLayout()
        main_box  = QVBoxLayout(self)

        self.file_fmt = QComboBox()
        self.file_fmt.addItems([
            ".bmp",
            ".dib",
            ".jpeg",
            ".jpg",
            ".jp",
            ".png",
            ".tiff", 
            ".tif"
            ])
        lbl = QLabel("File format")
        file_fmt_box = QVBoxLayout()
        file_fmt_box.addWidget(lbl)
        file_fmt_box.addWidget(self.file_fmt)

        self.wind_size = QLineEdit()
        self.wind_size.setText("64") 
        self.wind_size.setAlignment(Qt.AlignRight)
        self.wind_size.setValidator(QIntValidator(1, 256))
        lbl = QLabel("Window size")
        wind_size_box = QVBoxLayout()
        wind_size_box.addWidget(lbl)
        wind_size_box.addWidget(self.wind_size)

        self.overlap = QLineEdit()
        self.overlap.setText("32")
        self.overlap.setAlignment(Qt.AlignRight)
        self.overlap.setValidator(QIntValidator(1, 256))
        lbl = QLabel("Overlap")
        overlap_box = QVBoxLayout()
        overlap_box.addWidget(lbl)
        overlap_box.addWidget(self.overlap)

        self.iteration_mod = QComboBox()
        self.iteration_mod.addItems([
            "DWS",
            "CWS"
            ])
        lbl = QLabel("Iteration Mod")
        iteration_mod_box = QVBoxLayout()
        iteration_mod_box.addWidget(lbl)
        iteration_mod_box.addWidget(self.iteration_mod)

        self.device = QComboBox()
        available_gpus = [torch.cuda.get_device_name(i) for i in range(torch.cuda.device_count())]
        self.device.addItems(["cpu"]+available_gpus)
        lbl = QLabel("Device")
        device_box = QVBoxLayout()
        device_box.addWidget(lbl)
        device_box.addWidget(self.device)
        
        self.scale = QLineEdit()
        self.scale.setAlignment(Qt.AlignRight)
        self.scale.setText("0.0165364")
        validator = QDoubleValidator()
        validator.setLocale(QLocale("en_US"))
        self.scale.setValidator(validator)
        lbl = QLabel("scale mm/px")
        scale_box = QVBoxLayout()
        scale_box.addWidget(lbl)
        scale_box.addWidget(self.scale)

        self.time = QLineEdit()
        self.time.setText("11")
        self.time.setValidator(QIntValidator(1, 2147483647))
        self.time.setAlignment(Qt.AlignRight)
        lbl = QLabel("dt, us")
        time_box = QVBoxLayout()
        time_box.addWidget(lbl)
        time_box.addWidget(self.time)

        self.save_options = QComboBox()
        self.save_options.addItems([
            "Save statistics",
            "Save all",
            "Dont save",
        ])
        lbl = QLabel("Save options")
        save_box = QVBoxLayout()
        save_box.addWidget(lbl)
        save_box.addWidget(self.save_options)
        
        self.iteration_scale = QLineEdit()
        self.iteration_scale.setAlignment(Qt.AlignRight)
        self.iteration_scale.setText("2.0")
        validator = QDoubleValidator()
        validator.setLocale(QLocale("en_US"))
        self.iteration_scale.setValidator(validator)
        lbl = QLabel("Iter wind rescale")
        iter_scale_box = QVBoxLayout()
        iter_scale_box.addWidget(lbl)
        iter_scale_box.addWidget(self.iteration_scale)


        self.iteration = QLineEdit()
        self.iteration.setText("1")
        self.iteration.setValidator(QIntValidator(1, 10))
        self.iteration.setAlignment(Qt.AlignRight)
        lbl = QLabel("Iterations")
        iteration_box = QVBoxLayout()
        iteration_box.addWidget(lbl)
        iteration_box.addWidget(self.iteration)

        file_button = QPushButton("...")
        file_button.setFixedWidth(40)
        file_button.clicked.connect(self.open_dialog)
        lbl = QLabel()
        file_button_box = QVBoxLayout()
        file_button_box.addWidget(lbl)
        file_button_box.addWidget(file_button)

        self.save_folder = QTextEdit()
        self.save_folder.setFixedHeight(27)
        lbl = QLabel("Save directory")
        folder_box = QVBoxLayout()
        folder_box.addWidget(lbl)
        folder_box.addWidget(self.save_folder)

        self.confirm = QPushButton("Confirm")
        self.confirm.setFixedWidth(180)
        self.confirm.clicked.connect(self.confirm_changes)

        hbox_top.addLayout(wind_size_box)
        hbox_top.addLayout(overlap_box)
        hbox_top.addLayout(iteration_mod_box)
        hbox_top.addLayout(file_fmt_box)
        hbox_bot.addLayout(scale_box)
        hbox_bot.addLayout(time_box)
        hbox_bot.addLayout(save_box)
        hbox_bot.addLayout(device_box)


        hhbox = QHBoxLayout()
        
        hhbox.addLayout(iteration_box)
        hhbox.setAlignment(Qt.AlignRight)

        hhbox.addLayout(iter_scale_box)
        hhbox.setAlignment(Qt.AlignRight)



        hhbox.addLayout(folder_box)
        hhbox.addLayout(file_button_box)
        dummybox = QVBoxLayout()
        lbl = QLabel()
        self.confirm.setFixedWidth(140)
        dummybox.addWidget(lbl)
        dummybox.addWidget(self.confirm)
        dummybox.setAlignment(Qt.AlignRight)

        regimebox = QVBoxLayout()
        lbl = QLabel("PIV regime")
        self.regime_box = QComboBox()
        self.regime_box.addItems([
            "offline",
            "online"
        ])
        regimebox.addWidget(lbl)
        regimebox.addWidget(self.regime_box)

        
        hhhbox = QHBoxLayout()
        hhhbox.addLayout(regimebox)
        hhhbox.addLayout(dummybox)    
    
        vbox_down.addLayout(hhbox)
        vbox_down.addLayout(hhhbox)


        main_box.addLayout(hbox_top)
        main_box.addLayout(hbox_bot)
        main_box.addLayout(vbox_down)
        set_width(self, QLineEdit, 120)
        set_width(self, QComboBox, 180)
        self.set_valeues()

    def set_valeues(self):
        
        self.wind_size.setText(str(self.state.wind_size))
        self.overlap.setText(str(self.state.overlap))
        idx = self.iteration_mod.findText(str(self.state.iter_mod), Qt.MatchContains)
        if idx >=0: self.iteration_mod.setCurrentIndex(idx)
        idx = self.device.findText(self.state.device, Qt.MatchContains)
        if idx >= 0: self.device.setCurrentIndex(idx) 
        self.scale.setText(str(self.state.scale))
        self.time.setText(str(self.state.dt))
        idx = self.file_fmt.findText(self.state.file_fmt, Qt.MatchContains)
        if idx >= 0: self.file_fmt.setCurrentIndex(idx)
        idx = self.save_options.findText(self.state.save_opt, Qt.MatchContains)
        if idx >=0: self.save_options.setCurrentIndex(idx)
        self.save_folder.setText(self.state.save_dir)
        self.iteration.setText(str(self.state.iterations))
        self.iteration_scale.setText(str(self.state.iter_scale))
        idx = self.regime_box.findText(str(self.state.regime), Qt.MatchContains)
        if idx >=0: self.regime_box.setCurrentIndex(idx)
    
    
    def open_dialog(self, checked):
        folder = QFileDialog.getExistingDirectory()
        self.save_folder.setText(folder)

    def close(self) -> bool:
        self.set_valeues()
        return super().close()

    def confirm_changes(self, checked):
        
        self.state.wind_size = int(self.wind_size.text())
        self.state.overlap = int(self.overlap.text())
        self.state.iter_mod =self.iteration_mod.currentText()
        self.state.device   = self.device.currentText()
        self.state.scale    = float(self.scale.text())
        self.state.dt       = int(self.time.text())
        self.state.file_fmt = self.file_fmt.currentText()
        self.state.save_opt = self.save_options.currentText()
        self.state.save_dir = self.save_folder.toPlainText()
        self.state.iterations = int(self.iteration.text())
        self.state.iter_scale = float(self.iteration_scale.text())
        self.state.regime     = self.regime_box.currentText()
        self.state.to_json()
        if self.isVisible():
            self.hide()

class ViewSettings(QWidget):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__( *args, **kwargs)
        self.initUI()

    def initUI(self):
        self.pos_scale_slider = ListSlider(orientation=Qt.Horizontal)
        self.neg_scale_slider = ListSlider(orientation=Qt.Horizontal)
        self.pos_scale_slider.setFixedWidth(200)
        self.neg_scale_slider.setFixedWidth(200)
        self.pos_scale_slider.values = list(range(2000))
        self.pos_scale_slider.setValue(1499)
        self.neg_scale_slider.values = list(range(2000))
        self.neg_scale_slider.setValue(499)
        
        self.pos_scale_text = QLineEdit()
        validator = QDoubleValidator()
        validator.setLocale(QLocale("en_US"))
        self.pos_scale_text.setValidator(validator)
        self.neg_scale_text = QLineEdit()
        validator = QDoubleValidator()
        validator.setLocale(QLocale("en_US"))
        self.neg_scale_text.setValidator(validator)

        self.pos_scale_text.editingFinished.connect(self.on_posLineEditChanged)
        self.neg_scale_text.editingFinished.connect(self.on_negLineEditChanged)
        
        self.quiver_box = QCheckBox("Show Quivers")
        self.streamlines_box = QCheckBox("Show Streamlines")
        self.hide_line_box = QCheckBox("Show Profile Line")
        self.hide_line_box.toggle()
        self.axes_box = QCheckBox("Show Axes")
        self.grid_box = QCheckBox("Show Grid")


        pos_hbox = QHBoxLayout()
        pos_hbox.addWidget(self.pos_scale_slider)
        pos_hbox.addWidget(self.pos_scale_text)
        neg_hbox = QHBoxLayout()
        neg_hbox.addWidget(self.neg_scale_slider)
        neg_hbox.addWidget(self.neg_scale_text)
        
        checkbox_vbox = QVBoxLayout()
        checkbox_vbox.addWidget(self.streamlines_box)
        checkbox_vbox.addWidget(self.hide_line_box)
        checkbox_vbox.addWidget(self.quiver_box)
        checkbox_vbox.addWidget(self.axes_box)
        checkbox_vbox.addWidget(self.grid_box)



        layout = QVBoxLayout()
        layout.addLayout(pos_hbox)
        layout.addLayout(neg_hbox)
        layout.addLayout(checkbox_vbox)

        self.setLayout(layout)
    def on_posLineEditChanged(self):
        self.pos_scale_slider.set_value(self.pos_scale_text.text())
    def on_negLineEditChanged(self):
        self.neg_scale_slider.set_value(self.neg_scale_text.text())

class ProfileControls(QWidget):
    fieldchosen = pyqtSignal(str)
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.settings = ViewSettings()
        self.field_box = QComboBox()
        self.data = Database()
        self.initialized: bool = False
        self.setFixedHeight(120)
        self.hide_line_box = self.settings.hide_line_box
        self.streamlines_box = self.settings.streamlines_box
        self.grid_box = self.settings.grid_box
        self.axes_box = self.settings.axes_box



        self.initUI()
    def initUI(self):
        self.slider_LCD = QLCDNumber()
        self.slider_LCD.setFrameShape(QFrame.NoFrame)
        self.slider_LCD.setSegmentStyle(QLCDNumber.Flat)

        self.slider = ListSlider(orientation=Qt.Horizontal)
        self.orientation_qbox = QComboBox()
        self.orientation_qbox.setFixedWidth(100)
        slider_box = QHBoxLayout()
        slider_box.addWidget(self.slider)
        slider_box.addWidget(self.slider_LCD)
        slider_box.addWidget(self.orientation_qbox)
        bottom_right_box = QVBoxLayout()
        lbl = QLabel("Profile control:")
        bottom_right_box.addWidget(lbl)
        bottom_right_box.addLayout(slider_box)
        bottom_right_box.addWidget(self.field_box)
        
        bottom_right_frame = QFrame()
        bottom_right_frame.setLayout(bottom_right_box)
        bottom_right_frame.setLineWidth(1)
        bottom_right_frame.setFrameStyle(QFrame.Panel)

        main_box = QHBoxLayout()
        main_box.addWidget(bottom_right_frame)

        self.setLayout(main_box)
    

    @pyqtSlot(str)
    def on_activated(self, key):
        if key is None:
            return
        self.fieldchosen.emit(key)

    @pyqtSlot(str)
    def on_orientation(self, key):
        piv_data = self.data.get()
        if key == "Horizontal":
            self.slider.values = piv_data[[*piv_data.keys()][1]][:, 0]
        else:
            self.slider.values = piv_data[[*piv_data.keys()][0]][0]

        self.fieldchosen.emit(key)
        
    def open_dialog(self):
        name, check = QFileDialog.getOpenFileName()
        if not check:
            return
        self.data.load(name)
        self.set_field_box()
    
    def init_profile(self):
        self.field_box.activated[str].connect(self.on_activated)
        self.orientation_qbox.clear()
        self.orientation_qbox.addItems(["Horizontal", "Vertical"])
        self.orientation_qbox.activated[str].connect(self.on_orientation)


    def set_field_box(self):
        piv_data = self.data.get()
        self.field_box.clear()
        self.field_box.addItems([*piv_data.keys()][2:])
        self.field_box.setCurrentText("Vy[m/s]")
        self.slider.values = piv_data[[*piv_data.keys()][1]][:, 0]
        self.slider.setValue(0)
        self.init_profile()

    def show_settings(self, checked):
        if self.settings.isVisible():
            self.settings.hide()
        else:
            self.settings.show()

class AnalysControlWidget(QWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.settings = Settings()
        self.piv_button = QPushButton("Start PIV")
        self.pause_button = QPushButton("Pause")
        self.pbar = QProgressBar()
        self.piv_button.pressed.connect(self._changeButton)
        self.initUI()
    def initUI(self):
        # self.pbar.setStyleSheet("QProgressBar::chunk "
        #           "{"
        #             "background-color: green;"
        #             "border-style: outset;"
        #             "border-width: 2px;"
        #             "border-radius: 10px;"
        #             "border-color: beige;"
        #             "font: bold 14px;"
        #             "min-width: 10em;"
        #             "padding: 6px;"
        #           "}")

        progress_box = QVBoxLayout()
        lbl = QLabel("Total progress: ")
        progress_box.addWidget(lbl)
        progress_box.addWidget(self.pbar)
        pb_h = QHBoxLayout()
        pb_h.addWidget(self.pause_button)
        pb_h.addWidget(self.piv_button)
        progress_box.addLayout(pb_h)


        self.setLayout(progress_box)
        self.setFixedHeight(120)
        
    def open_dialog(self, checked):
        folder = QFileDialog.getExistingDirectory()
        self.settings.state.folder = folder
    
    def _changeButton(self):
        if self.piv_button.text() == "Start PIV":
            self.piv_button.setText("Stop PIV")
        else:
            self.piv_button.setText("Start PIV")

    def show_settings(self, checked):
        if self.settings.isVisible():
            self.settings.hide()
        else:
            self.settings.show()


import logging
import traceback
import warnings
warnings.filterwarnings("ignore")
import sys
import gc
from PyQt5.QtGui import QFont
from PyQt5.QtCore import QObject, QThread, pyqtSignal, QTimer
from PyQt5.QtWidgets import (
    QApplication,
    QMainWindow,
    QVBoxLayout,
    QWidget,
    QMessageBox,
    QAction,
    QMenu,
    QHBoxLayout,
)
from torchPIV.PIVwidgets import PIVWidget
from torchPIV.ControlsWidgets import AnalysControlWidget
from torchPIV.PlotterFunctions import Database, show_message
from torchPIV.workers import PIVWorker, OnlineWorker
gc.disable() 

class MainWindow(QMainWindow):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.piv_widget = PIVWidget()
        self.controls = AnalysControlWidget()
        self.data = Database()
        self.controls.piv_button.clicked.connect(self.start_piv)
        self.controls.pause_button.clicked.connect(self.pause_piv)
        self.controls.piv_button.clicked.connect(self.stop_piv)
        self.timer = QTimer()
        self.calc_thread = None
        self.worker = None
        self.timer.timeout.connect(self.piv_widget.piv_view.set_field)
        self.initUI()

    def initUI(self):
        layout = QVBoxLayout()
        layout.addWidget(self.piv_widget)
        h_layout = QHBoxLayout()
        h_layout.addWidget(self.controls)
        h_layout.addWidget(self.piv_widget.controls)
        layout.addLayout(h_layout)
        w = QWidget()
        w.setLayout(layout)
        self.setCentralWidget(w)
        
        menuBar = self.menuBar()
        exitAction = QAction("Exit", self)
        exitAction.triggered.connect(self.close)
        fileMenu = menuBar.addMenu("&File")
        settings = QAction("Analysis Settings", self)
        settings.triggered.connect(self.controls.show_settings)
        menuBar.addAction(settings)
        viewControls = QAction("View Settings", self)
        viewControls.triggered.connect(self.piv_widget.controls.show_settings)
        menuBar.addAction(viewControls)
        folderMenu = QMenu("PIV folder", self)
        selectFolder = QAction("&New", self)
        selectFolder.triggered.connect(self.controls.open_dialog)
        folderMenu.addAction(selectFolder)
        folderMenu.addAction(QAction(self.controls.settings.state.folder, self))
        pivfileMenu = QMenu("Open PIV file", self)
        pivfileAction = QAction("&New", self)
        pivfileAction.triggered.connect(self.piv_widget.controls.open_dialog)
        pivfileMenu.addAction(pivfileAction)

        
        saveMenu = QMenu("&Save", self)
        saveMenu.addActions([
            QAction("Save profile", self),
            QAction("Save colormap", self)
        ])
        fileMenu.addMenu(folderMenu)
        fileMenu.addMenu(pivfileMenu)
        fileMenu.addMenu(saveMenu)
        fileMenu.addAction(exitAction)

    def exit(self, checked):
        self.controls.close()
        exit()
        
    def reportOutput(self, output: dict):
        self.data.set(output)
        if not self.piv_widget.controls.initialized:
            self.piv_widget.controls.set_field_box()
            self.piv_widget.controls.initialized = True
            self.piv_widget.piv_view.set_key("Vy[m/s]")
            self.piv_widget.controls.field_box.setCurrentText("Vy[m/s]")

    def reportProgress(self, value):
        self.controls.pbar.setValue(value)

    def reportFail(self):
        show_message(f'Dataset is empty')
        self.timer.stop()
        self.calc_thread.quit()
        self.calc_thread.wait()
        self.worker = None
        self.controls.piv_button.setText("Start PIV")
        gc.collect()

    def reportFinish(self, output: dict):
        if self.controls.settings.state.save_opt != "Dont save":
            show_message(
                f'Averaged data saved in\n{self.controls.settings.state.save_dir}'
                )
        self.data.set(output)
        self.timer.stop()
        self.piv_widget.controls.set_field_box()
        self.piv_widget.piv_view.set_field()
        self.calc_thread.quit()
        self.calc_thread.wait()
        self.calc_thread = None
        self.worker = None
        self.controls.piv_button.setText("Start PIV")
        gc.collect()
    
    def pause_piv(self):
        if self.calc_thread is None:
            return
        if self.worker.is_paused:
            text = "Pause"
        else:
            text = "Resume"

        self.controls.pause_button.setText(text)
        self.worker.is_paused = not self.worker.is_paused

    def stop_piv(self):
        if self.controls.piv_button.text() == "Stop PIV":
            return
        if self.calc_thread is None:
            return
        self.worker.is_running = False
        self.controls.pbar.setValue(0)


    def start_piv(self):
        if self.controls.piv_button.text() == "Start PIV":
            return
    
        self.piv_widget.controls.initialized = False

        self.timer.start(2000)
        self.controls.settings.state.to_json()
        self.calc_thread = QThread()
        piv_params = self.controls.settings.state
        if self.controls.settings.regime_box.currentText() == "offline":
            self.worker = PIVWorker(piv_params=piv_params)
        elif self.controls.settings.regime_box.currentText() == "online":
            self.worker = OnlineWorker(piv_params=piv_params)

        # self.worker.is_running = True
        # Step 4: Move worker to the thread
        self.worker.moveToThread(self.calc_thread)
        # Step 5: Connect signals and slots
        self._disable_buttons()
        self.calc_thread.finished.connect(
            self._enable_buttons
        )
        self.calc_thread.started.connect(self.worker.run)
        self.worker.finished.connect(self.worker.deleteLater)
        self.calc_thread.finished.connect(self.worker.deleteLater)
        self.calc_thread.finished.connect(self.calc_thread.deleteLater)
        self.worker.output.connect(self.reportOutput)
        self.worker.progress.connect(self.reportProgress)
        self.worker.finished.connect(self.reportFinish)
        self.worker.failed.connect(self.reportFail)
        # Step 6: Start the thread
        self.calc_thread.start()

        # Final resets
        
    def _disable_buttons(self):
        self.controls.settings.confirm.setEnabled(False)

    
    def _enable_buttons(self):
        self.controls.settings.confirm.setEnabled(True)

    def message(self, string: str):
        print(string)


# basic logger functionality
log = logging.getLogger(__name__)
handler = logging.StreamHandler(stream=sys.stdout)
log.addHandler(handler)

def show_exception_box(log_msg):
    """Checks if a QApplication instance is available and shows a messagebox with the exception message. 
    If unavailable (non-console application), log an additional notice.
    """
    #NOT IMPLEMENTED
    def onclick(button):
        if button.text() == "OK":
            QApplication.exit()
        elif button.text() == "Retry":
            pass


    if QApplication.instance() is not None:
            errorbox = QMessageBox()
            errorbox.setIcon(QMessageBox.Critical)
            errorbox.setText(f"Oops. An unexpected error occured:\n{log_msg}")
            errorbox.setStandardButtons(QMessageBox.Ok)
            errorbox.buttonClicked.connect(onclick)
            errorbox.exec_()
    else:
        log.debug("No QApplication instance available.")


 
class UncaughtHook(QObject):
    _exception_caught = pyqtSignal(object)
 
    def __init__(self, *args, **kwargs):
        super(UncaughtHook, self).__init__(*args, **kwargs)

        # this registers the exception_hook() function as hook with the Python interpreter
        sys.excepthook = self.exception_hook

        # connect signal to execute the message box function always on main thread
        self._exception_caught.connect(show_exception_box)
 
    def exception_hook(self, exc_type, exc_value, exc_traceback):
        """Function handling uncaught exceptions.
        It is triggered each time an uncaught exception occurs. 
        """
        if issubclass(exc_type, KeyboardInterrupt):
            # ignore keyboard interrupt to support console applications
            sys.__excepthook__(exc_type, exc_value, exc_traceback)
        else:
            exc_info = (exc_type, exc_value, exc_traceback)
            log_msg = '\n'.join([''.join(traceback.format_tb(exc_traceback)),
                                 '{0}: {1}'.format(exc_type.__name__, exc_value)])
            log.critical("Uncaught exception:\n {0}".format(log_msg), exc_info=exc_info)

            # trigger message box show
            self._exception_caught.emit(log_msg)
 
# create a global instance of our class to register the hook
qt_exception_hook = UncaughtHook()


def runGUI():
    app = QApplication(sys.argv)
    app.setStyle("fusion")
    app.setFont(QFont("Helvetica", 12))
    w = MainWindow()
    w.show()
    sys.exit(app.exec_())


if __name__ == "__main__":
    runGUI()

